'use strict';

var react = require('react');

function _typeof(obj) {
  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var isCheckBoxInput = (function (type) {
  return type === 'checkbox';
});

function attachEventListeners(_ref) {
  var field = _ref.field,
      validateAndStateUpdate = _ref.validateAndStateUpdate,
      isRadio = _ref.isRadio;
  var ref = field.ref;
  if (!ref.addEventListener) return;
  ref.addEventListener(isCheckBoxInput(ref.type) || isRadio ? 'change' : 'input', validateAndStateUpdate);
  ref.addEventListener('blur', validateAndStateUpdate);
}

var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/;
var reIsPlainProp = /^\w*$/;
var rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;
var reEscapeChar = /\\(\\)?/g;
var reIsUint = /^(?:0|[1-9]\d*)$/;
var isArray = Array.isArray;

function isIndex(value) {
  return reIsUint.test(value) && value > -1;
}

function isKey(value) {
  if (isArray(value)) return false;
  return reIsPlainProp.test(value) || !reIsDeepProp.test(value);
}

var stringToPath = function stringToPath(string) {
  var result = [];
  string.replace(rePropName, function (match, number, quote, string) {
    result.push(quote ? string.replace(reEscapeChar, '$1') : number || match);
  });
  return result;
};

function set(object, path, value) {
  path = isKey(path) ? [path] : stringToPath(path);
  var index = -1;
  var length = path.length;
  var lastIndex = length - 1;

  while (++index < length) {
    var key = path[index];
    var newValue = value;

    if (index != lastIndex) {
      var objValue = object[key];
      var isObject = _typeof(objValue) === 'object';
      newValue = isObject ? objValue : isIndex(path[index + 1]) ? [] : {};
    }

    object[key] = newValue;
    object = object[key];
  }

  return object;
}

function combineFieldValues(data) {
  var output = Object.entries(data).reduce(function (previous, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        value = _ref2[1];

    var isArray = key.match(/\[\d+\]/gi);
    var isObject = key.indexOf('.') >= 0;

    if (isArray || isObject) {
      set(previous, key, value);
      return previous;
    }

    previous[key] = value;
    return previous;
  }, {});
  return Object.entries(output).reduce(function (previous, _ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        key = _ref4[0],
        value = _ref4[1];

    if (Array.isArray(value)) {
      previous[key] = value.filter(Boolean);
      return previous;
    }

    previous[key] = value;
    return previous;
  }, {});
}

var removeAllEventListeners = (function (ref, validateWithStateUpdate) {
  if (!ref.removeEventListener) return;
  ref.removeEventListener('input', validateWithStateUpdate);
  ref.removeEventListener('change', validateWithStateUpdate);
  ref.removeEventListener('blur', validateWithStateUpdate);
});

var isRadioInput = (function (type) {
  return type === 'radio';
});

function findRemovedFieldAndRemoveListener(fields, validateWithStateUpdate, _ref) {
  var ref = _ref.ref,
      mutationWatcher = _ref.mutationWatcher,
      options = _ref.options;
  var forceDelete = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
  if (!ref || !ref.type) return;
  var name = ref.name,
      type = ref.type;

  if (isRadioInput(type) && options) {
    options.forEach(function (_ref2, index) {
      var ref = _ref2.ref;

      if (ref instanceof HTMLElement && !document.body.contains(ref) && options && options[index]) {
        removeAllEventListeners(options[index], validateWithStateUpdate);
        (options[index].mutationWatcher || {
          disconnect: function disconnect() {}
        }).disconnect();
        options.splice(index, 1);
      }
    });

    if (Array.isArray(options) && !options.length) {
      delete fields[name];
      return fields;
    }
  } else if (ref instanceof HTMLElement && !document.body.contains(ref) || forceDelete) {
    removeAllEventListeners(ref, validateWithStateUpdate);
    if (mutationWatcher) mutationWatcher.disconnect();
    delete fields[name];
    return fields;
  }

  return fields;
}

var defaultReturn = {
  isValid: false,
  value: ''
};
function getRadioValue(options) {
  return Array.isArray(options) ? options.reduce(function (previous, _ref) {
    var _ref$ref = _ref.ref,
        checked = _ref$ref.checked,
        value = _ref$ref.value;
    return checked ? {
      isValid: true,
      value: value
    } : previous;
  }, defaultReturn) : defaultReturn;
}

var getMultipleSelectValue = (function (options) {
  return options.filter(function (_ref) {
    var selected = _ref.selected;
    return selected;
  }).map(function (_ref2) {
    var value = _ref2.value;
    return value;
  });
});

function getFieldValue(fields, _ref) {
  var type = _ref.type,
      name = _ref.name,
      options = _ref.options,
      checked = _ref.checked,
      value = _ref.value;

  if (isRadioInput(type)) {
    var fieldValue = fields[name];
    if (!fieldValue) throw new Error("Expected ".concat(name, " field is missing"));
    return getRadioValue(fieldValue.options).value;
  }

  if (type === 'select-multiple') return getMultipleSelectValue(options);
  if (isCheckBoxInput(type)) return checked ? value || checked : false;
  return value;
}

var isString = (function (value) {
  return value && typeof value === 'string';
});

function getFieldsValues(fields, fieldName) {
  return Object.values(fields).reduce(function (previous, data) {
    var ref = data.ref,
        name = data.ref.name;
    var value = getFieldValue(fields, ref);

    if (isString(fieldName)) {
      return name === fieldName ? value : previous;
    }

    if (Array.isArray(fieldName)) {
      if (fieldName.includes(name)) {
        previous[name] = value;
      }
    } else {
      previous[name] = value;
    }

    return previous;
  }, {});
}

var isEmptyObject = (function (values) {
  return values && values.constructor === Object && Object.keys(values).length === 0;
});

function shouldUpdateWithError(_ref) {
  var errors = _ref.errors,
      name = _ref.name,
      error = _ref.error,
      isOnBlur = _ref.isOnBlur,
      isBlurType = _ref.isBlurType,
      validateDisabled = _ref.validateDisabled;

  if (validateDisabled || isOnBlur && !isBlurType || isEmptyObject(error) && isEmptyObject(errors)) {
    return false;
  }

  if (isEmptyObject(errors) && !isEmptyObject(error) || isEmptyObject(error) && errors[name]) {
    return true;
  }

  if (!errors[name]) {
    return true;
  } else if (errors[name].type !== error[name].type || errors[name].message !== error[name].message) {
    return true;
  }

  return false;
}

var DATE_INPUTS = ['date', 'time', 'month', 'datetime', 'datetime-local', 'week'];
var STRING_INPUTS = ['text', 'email', 'password', 'search', 'tel', 'url', 'textarea'];
var VALIDATION_MODE = {
  onBlur: 'onBlur',
  onchange: 'onChange',
  onSubmit: 'onSubmit'
};

var getValueAndMessage = (function (item) {
  return {
    value: _typeof(item) === 'object' && item.value ? item.value : item,
    message: _typeof(item) === 'object' && item.message ? item.message : ''
  };
});

var validateField = /*#__PURE__*/
(function () {
  var _ref2 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(_ref, fields) {
    var ref, _ref$ref, type, value, name, checked, options, required, maxLength, minLength, min, max, pattern, validate, error, isRadio, isCheckBox, isSelectOrInput, exceedMax, exceedMin, valueNumber, _getValueAndMessage, maxValue, maxMessage, _getValueAndMessage2, minValue, minMessage, _getValueAndMessage3, maxLengthValue, maxLengthMessage, _getValueAndMessage4, minLengthValue, minLengthMessage, _exceedMax, _exceedMin, _getValueAndMessage5, patternValue, patternMessage, fieldValue, validateRef, result, validationResult;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            ref = _ref.ref, _ref$ref = _ref.ref, type = _ref$ref.type, value = _ref$ref.value, name = _ref$ref.name, checked = _ref$ref.checked, options = _ref.options, required = _ref.required, maxLength = _ref.maxLength, minLength = _ref.minLength, min = _ref.min, max = _ref.max, pattern = _ref.pattern, validate = _ref.validate;
            error = {};
            isRadio = isRadioInput(type);
            isCheckBox = isCheckBoxInput(type);
            isSelectOrInput = !isCheckBox && !isRadio;

            if (!(required && (isCheckBox && !checked || isSelectOrInput && value === '' || isRadio && !getRadioValue(fields[name].options).isValid || !type && !value))) {
              _context2.next = 8;
              break;
            }

            error[name] = {
              type: 'required',
              message: isString(required) ? required : '',
              ref: isRadio ? (fields[name].options || [{
                ref: ''
              }])[0].ref : ref
            };
            return _context2.abrupt("return", error);

          case 8:
            if (!((min || max) && !STRING_INPUTS.includes(type))) {
              _context2.next = 16;
              break;
            }

            valueNumber = parseFloat(value);
            _getValueAndMessage = getValueAndMessage(max), maxValue = _getValueAndMessage.value, maxMessage = _getValueAndMessage.message;
            _getValueAndMessage2 = getValueAndMessage(min), minValue = _getValueAndMessage2.value, minMessage = _getValueAndMessage2.message;

            if (type === 'number') {
              exceedMax = maxValue && valueNumber > maxValue;
              exceedMin = minValue && valueNumber < minValue;
            } else if (DATE_INPUTS.includes(type)) {
              if (typeof maxValue === 'string') exceedMax = maxValue && new Date(value) > new Date(maxValue);
              if (typeof minValue === 'string') exceedMin = minValue && new Date(value) < new Date(minValue);
            }

            if (!(exceedMax || exceedMin)) {
              _context2.next = 16;
              break;
            }

            error[name] = Object.assign({}, error[name], {
              type: exceedMax ? 'max' : 'min',
              message: exceedMax ? maxMessage : minMessage,
              ref: ref
            });
            return _context2.abrupt("return", error);

          case 16:
            if (!((maxLength || minLength) && STRING_INPUTS.includes(type))) {
              _context2.next = 24;
              break;
            }

            _getValueAndMessage3 = getValueAndMessage(maxLength), maxLengthValue = _getValueAndMessage3.value, maxLengthMessage = _getValueAndMessage3.message;
            _getValueAndMessage4 = getValueAndMessage(minLength), minLengthValue = _getValueAndMessage4.value, minLengthMessage = _getValueAndMessage4.message;
            _exceedMax = maxLength && value.toString().length > maxLengthValue;
            _exceedMin = minLength && value.toString().length < minLengthValue;

            if (!(_exceedMax || _exceedMin)) {
              _context2.next = 24;
              break;
            }

            error[name] = Object.assign({}, error[name], {
              type: _exceedMax ? 'maxLength' : 'minLength',
              message: _exceedMax ? maxLengthMessage : minLengthMessage,
              ref: ref
            });
            return _context2.abrupt("return", error);

          case 24:
            if (!pattern) {
              _context2.next = 29;
              break;
            }

            _getValueAndMessage5 = getValueAndMessage(pattern), patternValue = _getValueAndMessage5.value, patternMessage = _getValueAndMessage5.message;

            if (!(patternValue instanceof RegExp && !patternValue.test(value))) {
              _context2.next = 29;
              break;
            }

            error[name] = Object.assign({}, error[name], {
              type: 'pattern',
              message: patternMessage,
              ref: ref
            });
            return _context2.abrupt("return", error);

          case 29:
            if (!validate) {
              _context2.next = 53;
              break;
            }

            fieldValue = isRadio ? getRadioValue(options).value : value;
            validateRef = isRadio && options ? options[0].ref : ref;

            if (!(typeof validate === 'function')) {
              _context2.next = 46;
              break;
            }

            _context2.next = 35;
            return validate(fieldValue);

          case 35:
            result = _context2.sent;

            if (!(typeof result === 'string' && result)) {
              _context2.next = 41;
              break;
            }

            error[name] = Object.assign({}, error[name], {
              type: 'validate',
              message: result,
              ref: validateRef
            });
            return _context2.abrupt("return", error);

          case 41:
            if (!(typeof result === 'boolean' && !result)) {
              _context2.next = 44;
              break;
            }

            error[name] = Object.assign({}, error[name], {
              type: 'validate',
              message: '',
              ref: validateRef
            });
            return _context2.abrupt("return", error);

          case 44:
            _context2.next = 53;
            break;

          case 46:
            if (!(_typeof(validate) === 'object')) {
              _context2.next = 53;
              break;
            }

            _context2.next = 49;
            return new Promise(function (resolve) {
              var values = Object.entries(validate);
              values.reduce(
              /*#__PURE__*/
              function () {
                var _ref4 = _asyncToGenerator(
                /*#__PURE__*/
                regeneratorRuntime.mark(function _callee(previous, _ref3, index) {
                  var _ref5, key, validate, lastChild, _result, data;

                  return regeneratorRuntime.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          _ref5 = _slicedToArray(_ref3, 2), key = _ref5[0], validate = _ref5[1];
                          lastChild = values.length - 1 === index;

                          if (!(typeof validate === 'function')) {
                            _context.next = 9;
                            break;
                          }

                          _context.next = 5;
                          return validate(fieldValue);

                        case 5:
                          _result = _context.sent;

                          if (!(typeof _result !== 'boolean' || !_result)) {
                            _context.next = 9;
                            break;
                          }

                          data = {
                            type: key,
                            message: isString(_result) ? _result : '',
                            ref: validateRef
                          };
                          return _context.abrupt("return", lastChild ? resolve(data) : data);

                        case 9:
                          return _context.abrupt("return", lastChild ? resolve(previous) : previous);

                        case 10:
                        case "end":
                          return _context.stop();
                      }
                    }
                  }, _callee);
                }));

                return function (_x3, _x4, _x5) {
                  return _ref4.apply(this, arguments);
                };
              }(), {});
            });

          case 49:
            validationResult = _context2.sent;

            if (!(validationResult && !isEmptyObject(validationResult))) {
              _context2.next = 53;
              break;
            }

            error[name] = Object.assign({}, error[name], {
              ref: validateRef
            }, validationResult);
            return _context2.abrupt("return", error);

          case 53:
            return _context2.abrupt("return", error);

          case 54:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function (_x, _x2) {
    return _ref2.apply(this, arguments);
  };
})();

function parseErrorSchema(error) {
  return error.inner.reduce(function (previous, current, index) {
    previous[current.path] = error.errors[index];
    return previous;
  }, {});
}
function validateWithSchema(_x, _x2) {
  return _validateWithSchema.apply(this, arguments);
}

function _validateWithSchema() {
  _validateWithSchema = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(ValidationSchema, data) {
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return ValidationSchema.validate(data, {
              abortEarly: false
            });

          case 3:
            return _context.abrupt("return", {});

          case 6:
            _context.prev = 6;
            _context.t0 = _context["catch"](0);
            return _context.abrupt("return", parseErrorSchema(_context.t0));

          case 9:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[0, 6]]);
  }));
  return _validateWithSchema.apply(this, arguments);
}

var filterUndefinedErrors = (function (errors) {
  return Object.entries(errors).reduce(function (previous, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        value = _ref2[1];

    if (value && value.type) previous[key] = value;
    return previous;
  }, {});
});

function onDomRemove(element, onDetachCallback) {
  var observer = new MutationObserver(function () {
    function isDetached(element) {
      if (!element || !element.parentNode) return true;

      if (element.parentNode === window.document) {
        return false;
      } else if (element.parentNode === null) {
        return true;
      }

      return isDetached(element.parentNode);
    }

    if (isDetached(element)) {
      observer.disconnect();
      onDetachCallback();
    }
  });
  observer.observe(window.document, {
    childList: true,
    subtree: true
  });
  return observer;
}

var modeChecker = (function (mode) {
  return {
    isOnSubmit: !mode || mode === VALIDATION_MODE.onSubmit,
    isOnBlur: mode === VALIDATION_MODE.onBlur,
    isOnChange: mode === VALIDATION_MODE.onchange
  };
});

var warnMissingRef = (function (payload) {
  return console.warn("Oops missing the name for field: ".concat(payload));
});

function useForm() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    mode: 'onSubmit',
    defaultValues: {}
  },
      mode = _ref.mode,
      validationSchema = _ref.validationSchema,
      defaultValues = _ref.defaultValues,
      validationFields = _ref.validationFields;

  var fieldsRef = react.useRef({});
  var errorsRef = react.useRef({});
  var submitCountRef = react.useRef(0);
  var touchedFieldsRef = react.useRef([]);
  var watchFieldsRef = react.useRef({});
  var isWatchAllRef = react.useRef(false);
  var isSubmittingRef = react.useRef(false);
  var isSubmittedRef = react.useRef(false);
  var isDirtyRef = react.useRef(false);
  var reRenderForm = react.useState({})[1];
  var validateAndStateUpdateRef = react.useRef();

  var renderBaseOnError = function renderBaseOnError(name, errors, error) {
    if (errors[name] && !error[name]) {
      delete errorsRef.current[name];
      reRenderForm({});
      return true;
    } else if (error[name]) {
      reRenderForm({});
      return true;
    }

    return false;
  };

  var setValue = function setValue(name, value) {
    var shouldRender = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
    var field = fieldsRef.current[name];
    if (!field) return;

    if (shouldRender) {
      if (!touchedFieldsRef.current.includes(name)) {
        touchedFieldsRef.current.push(name);
      }

      isDirtyRef.current = true;
    }

    var ref = field.ref;
    var options = field.options;

    if (isRadioInput(ref.type) && options) {
      options.forEach(function (_ref2) {
        var radioRef = _ref2.ref;
        if (radioRef.value === value) radioRef.checked = true;
      });
      return;
    }

    ref[isCheckBoxInput(ref.type) ? 'checked' : 'value'] = value;
    if (shouldRender) reRenderForm({});
  };

  var isValidateDisabled = function isValidateDisabled() {
    return !isSubmittedRef.current && modeChecker(mode).isOnSubmit;
  };

  var triggerValidation =
  /*#__PURE__*/
  function () {
    var _ref4 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee(_ref3) {
      var name, value, forceValidation, field, errors, error;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              name = _ref3.name, value = _ref3.value, forceValidation = _ref3.forceValidation;
              field = fieldsRef.current[name];
              errors = errorsRef.current;

              if (field) {
                _context.next = 5;
                break;
              }

              return _context.abrupt("return", false);

            case 5:
              if (!(isValidateDisabled() && !forceValidation)) {
                _context.next = 7;
                break;
              }

              return _context.abrupt("return", isEmptyObject(errors));

            case 7:
              if (value !== undefined) setValue(name, value);
              _context.next = 10;
              return validateField(field, fieldsRef.current);

            case 10:
              error = _context.sent;
              errorsRef.current = Object.assign({}, filterUndefinedErrors(errorsRef.current), error);
              renderBaseOnError(name, errors, error);
              return _context.abrupt("return", isEmptyObject(error));

            case 14:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function triggerValidation(_x) {
      return _ref4.apply(this, arguments);
    };
  }();

  validateAndStateUpdateRef.current = validateAndStateUpdateRef.current ? validateAndStateUpdateRef.current :
  /*#__PURE__*/
  function () {
    var _ref6 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee2(_ref5) {
      var name, type, fields, errors, ref, isBlurType, _modeChecker, isOnChange, isOnBlur, validateDisabled, isWatchAll, shouldUpdateWatchMode, shouldUpdateValidateMode, shouldUpdateState, result, schemaValidateErrors, error, shouldUpdate, _error, _shouldUpdate;

      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              name = _ref5.target.name, type = _ref5.type;

              if (!(Array.isArray(validationFields) && !validationFields.includes(name))) {
                _context2.next = 3;
                break;
              }

              return _context2.abrupt("return");

            case 3:
              fields = fieldsRef.current;
              errors = errorsRef.current;
              ref = fields[name];

              if (ref) {
                _context2.next = 8;
                break;
              }

              return _context2.abrupt("return");

            case 8:
              isBlurType = type === 'blur';
              _modeChecker = modeChecker(mode), isOnChange = _modeChecker.isOnChange, isOnBlur = _modeChecker.isOnBlur;
              validateDisabled = isValidateDisabled();
              isWatchAll = isWatchAllRef.current;
              shouldUpdateWatchMode = isWatchAll || watchFieldsRef.current[name];
              shouldUpdateValidateMode = isOnChange || isOnBlur && isBlurType;
              shouldUpdateState = shouldUpdateWatchMode;

              if (!isDirtyRef.current) {
                isDirtyRef.current = true;
                shouldUpdateState = true;
              }

              if (!touchedFieldsRef.current.includes(name)) {
                touchedFieldsRef.current.push(name);
                shouldUpdateState = true;
              }

              if (!(validateDisabled && shouldUpdateWatchMode)) {
                _context2.next = 19;
                break;
              }

              return _context2.abrupt("return", reRenderForm({}));

            case 19:
              if (!validationSchema) {
                _context2.next = 35;
                break;
              }

              result = getFieldsValues(fields);
              _context2.next = 23;
              return validateWithSchema(validationSchema, result);

            case 23:
              _context2.t0 = _context2.sent;

              if (_context2.t0) {
                _context2.next = 26;
                break;
              }

              _context2.t0 = {};

            case 26:
              schemaValidateErrors = _context2.t0;
              error = schemaValidateErrors[name];
              shouldUpdate = (!error && errors[name] || error) && shouldUpdateValidateMode;

              if (!(shouldUpdate || shouldUpdateWatchMode)) {
                _context2.next = 33;
                break;
              }

              errorsRef.current = Object.assign({}, errors, _defineProperty({}, name, error));
              if (!error) delete errorsRef.current[name];
              return _context2.abrupt("return", reRenderForm({}));

            case 33:
              _context2.next = 43;
              break;

            case 35:
              _context2.next = 37;
              return validateField(ref, fields);

            case 37:
              _error = _context2.sent;
              _shouldUpdate = shouldUpdateWithError({
                errors: errors,
                error: _error,
                validateDisabled: validateDisabled,
                isOnBlur: isOnBlur,
                isBlurType: isBlurType,
                name: name
              });

              if (!(_shouldUpdate || shouldUpdateValidateMode || shouldUpdateWatchMode)) {
                _context2.next = 43;
                break;
              }

              errorsRef.current = Object.assign({}, filterUndefinedErrors(errors), _error);

              if (!renderBaseOnError(name, errorsRef.current, _error)) {
                _context2.next = 43;
                break;
              }

              return _context2.abrupt("return");

            case 43:
              if (shouldUpdateState) reRenderForm({});

            case 44:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function (_x2) {
      return _ref6.apply(this, arguments);
    };
  }();
  var removeEventListener = findRemovedFieldAndRemoveListener.bind(null, fieldsRef.current, validateAndStateUpdateRef.current);

  var setError = function setError(name, type, message, ref) {
    var errors = errorsRef.current;
    var error = errors[name];

    if (!type && error) {
      delete errors[name];
      reRenderForm({});
    } else if (type && error && (error.type !== type || error.message !== message)) {
      errors[name] = {
        type: type,
        message: message,
        ref: ref
      };
      reRenderForm({});
    }
  };

  function registerIntoFieldsRef(elementRef, data) {
    if (elementRef && !elementRef.name) return warnMissingRef(elementRef);
    var name = elementRef.name,
        type = elementRef.type,
        value = elementRef.value;

    var _ref7 = data || {},
        _ref7$required = _ref7.required,
        required = _ref7$required === void 0 ? false : _ref7$required,
        _ref7$validate = _ref7.validate,
        validate = _ref7$validate === void 0 ? undefined : _ref7$validate;

    var inputData = Object.assign({}, data, {
      ref: elementRef
    });
    var fields = fieldsRef.current;
    var isRadio = isRadioInput(type);
    var field = fields[name];
    var existRadioOptionIndex = isRadio && field && Array.isArray(field.options) ? field.options.findIndex(function (_ref8) {
      var ref = _ref8.ref;
      return value === ref.value;
    }) : -1;
    if (!isRadio && field || isRadio && existRadioOptionIndex > -1) return;

    if (!type) {
      fields[name] = Object.assign({
        ref: {
          name: name
        }
      }, data);
    } else {
      if (isRadio) {
        if (!field) fields[name] = {
          options: [],
          required: required,
          validate: validate,
          ref: {
            type: 'radio',
            name: name
          }
        };
        if (validate) fields[name].validate = validate;
        (fields[name].options || []).push(Object.assign({}, inputData, {
          mutationWatcher: onDomRemove(elementRef, function () {
            return removeEventListener(inputData, true);
          })
        }));
      } else {
        fields[name] = Object.assign({}, inputData, {
          mutationWatcher: onDomRemove(elementRef, function () {
            return removeEventListener(inputData, true);
          })
        });
      }
    }

    if (defaultValues && defaultValues[name]) {
      setValue(name, defaultValues[name], false);
    }

    if (!type) {
      return;
    }

    var fieldData = isRadio ? (fields[name].options || [])[(fields[name].options || []).length - 1] : fields[name];
    if (!fieldData) return;
    attachEventListeners({
      field: fieldData,
      isRadio: isRadio,
      validateAndStateUpdate: validateAndStateUpdateRef.current
    });
  }

  function watch(fieldNames, defaultValue) {
    var watchFields = watchFieldsRef.current;

    if (typeof fieldNames === 'string') {
      watchFields[fieldNames] = true;
    } else if (Array.isArray(fieldNames)) {
      fieldNames.forEach(function (name) {
        watchFields[name] = true;
      });
    } else {
      isWatchAllRef.current = true;
      watchFieldsRef.current = {};
    }

    var values = getFieldsValues(fieldsRef.current, fieldNames);
    var result = values === undefined || isEmptyObject(values) ? undefined : values;
    return result === undefined ? defaultValue : result;
  }

  var register = react.useCallback(function (refOrValidateRule, validateRule) {
    if (!refOrValidateRule || typeof window === 'undefined') return;

    if (validateRule && !refOrValidateRule.name) {
      warnMissingRef(refOrValidateRule);
      return;
    }

    if (refOrValidateRule.name) {
      registerIntoFieldsRef(refOrValidateRule, validateRule);
    }

    return function (ref) {
      return ref && registerIntoFieldsRef(ref, refOrValidateRule);
    };
  }, []);

  var handleSubmit = function handleSubmit(callback) {
    return (
      /*#__PURE__*/
      function () {
        var _ref9 = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee4(e) {
          var fieldErrors, fieldValues, fields, currentFieldValues, _ref10, errors, values;

          return regeneratorRuntime.wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  if (e) {
                    e.preventDefault();
                    e.persist();
                  }

                  fields = fieldsRef.current;
                  currentFieldValues = validationFields ? validationFields.map(function (name) {
                    return fieldsRef.current[name];
                  }) : Object.values(fields);
                  isSubmittingRef.current = true;
                  reRenderForm({});

                  if (!validationSchema) {
                    _context4.next = 12;
                    break;
                  }

                  fieldValues = getFieldsValues(fields);
                  _context4.next = 9;
                  return validateWithSchema(validationSchema, fieldValues);

                case 9:
                  fieldErrors = _context4.sent;
                  _context4.next = 19;
                  break;

                case 12:
                  _context4.next = 14;
                  return currentFieldValues.reduce(
                  /*#__PURE__*/
                  function () {
                    var _ref11 = _asyncToGenerator(
                    /*#__PURE__*/
                    regeneratorRuntime.mark(function _callee3(previous, field) {
                      var resolvedPrevious, ref, name, fieldError;
                      return regeneratorRuntime.wrap(function _callee3$(_context3) {
                        while (1) {
                          switch (_context3.prev = _context3.next) {
                            case 0:
                              _context3.next = 2;
                              return previous;

                            case 2:
                              resolvedPrevious = _context3.sent;
                              ref = field.ref, name = field.ref.name;

                              if (fields[name]) {
                                _context3.next = 6;
                                break;
                              }

                              return _context3.abrupt("return", Promise.resolve(resolvedPrevious));

                            case 6:
                              _context3.next = 8;
                              return validateField(field, fields);

                            case 8:
                              fieldError = _context3.sent;

                              if (!fieldError[name]) {
                                _context3.next = 12;
                                break;
                              }

                              resolvedPrevious.errors = Object.assign({}, resolvedPrevious.errors || {}, fieldError);
                              return _context3.abrupt("return", Promise.resolve(resolvedPrevious));

                            case 12:
                              resolvedPrevious.values[name] = getFieldValue(fields, ref);
                              return _context3.abrupt("return", Promise.resolve(resolvedPrevious));

                            case 14:
                            case "end":
                              return _context3.stop();
                          }
                        }
                      }, _callee3);
                    }));

                    return function (_x4, _x5) {
                      return _ref11.apply(this, arguments);
                    };
                  }(), Promise.resolve({
                    errors: {},
                    values: {}
                  }));

                case 14:
                  _ref10 = _context4.sent;
                  errors = _ref10.errors;
                  values = _ref10.values;
                  fieldErrors = Object.assign({}, errors, filterUndefinedErrors(errorsRef.current));
                  fieldValues = values;

                case 19:
                  if (!isEmptyObject(fieldErrors)) {
                    _context4.next = 24;
                    break;
                  }

                  _context4.next = 22;
                  return callback(combineFieldValues(fieldValues), e);

                case 22:
                  _context4.next = 25;
                  break;

                case 24:
                  errorsRef.current = fieldErrors;

                case 25:
                  isSubmittedRef.current = true;
                  submitCountRef.current += 1;
                  isSubmittingRef.current = false;
                  reRenderForm({});

                case 29:
                case "end":
                  return _context4.stop();
              }
            }
          }, _callee4);
        }));

        return function (_x3) {
          return _ref9.apply(this, arguments);
        };
      }()
    );
  };

  var unSubscribe = function unSubscribe() {
    fieldsRef.current && Object.values(fieldsRef.current).forEach(function (field) {
      var ref = field.ref,
          options = field.options;
      isRadioInput(ref.type) && Array.isArray(options) ? options.forEach(function (fieldRef) {
        return removeEventListener(fieldRef, true);
      }) : removeEventListener(field, true);
    });
    fieldsRef.current = {};
    watchFieldsRef.current = {};
    errorsRef.current = {};
    isWatchAllRef.current = false;
    isSubmittedRef.current = false;
    isDirtyRef.current = false;
    touchedFieldsRef.current = [];
  };

  var reset = function reset() {
    try {
      // @ts-ignore
      Object.values(fieldsRef.current)[0].ref.closest('form').reset();
    } catch (_a) {
      console.warn("\u26A0 No HTML input found, hence <form> look up failed.");
    }

    unSubscribe();
    reRenderForm({});
  };

  var getValues = function getValues() {
    return getFieldsValues(fieldsRef.current);
  };

  react.useEffect(function () {
    return unSubscribe;
  }, [mode]);
  return {
    register: register,
    handleSubmit: handleSubmit,
    watch: watch,
    unSubscribe: unSubscribe,
    reset: reset,
    setError: setError,
    setValue: setValue,
    triggerValidation: triggerValidation,
    getValues: getValues,
    errors: errorsRef.current,
    formState: {
      dirty: isDirtyRef.current,
      isSubmitted: isSubmittedRef.current,
      submitCount: submitCountRef.current,
      touched: touchedFieldsRef.current,
      isSubmitting: isSubmittingRef.current
    }
  };
}

module.exports = useForm;
